import os
import re
import subprocess
from textureminer.common import DEFAULT_SCALE_FACTOR, REGEX_BEDROCK_PREVIEW, REGEX_BEDROCK_RELEASE, EditionType, VersionType, filter_unwanted, rm_if_exists, tabbed_print, scale_textures, DEFAULT_OUTPUT_DIR, TEMP_PATH, validate_version
from textureminer import texts

REPO_URL = 'https://github.com/Mojang/bedrock-samples'


def get_version_type(version: str) -> VersionType:
    """Gets the type of a version based on regex.

    Args:
        version (str): version to get the type of

    Returns:
        VersionType: type of version
    """
    if version[0] != 'v':
        version = f'v{version}'
    if re.match(REGEX_BEDROCK_RELEASE, version):
        return VersionType.RELEASE
    if re.match(REGEX_BEDROCK_PREVIEW, version):
        return VersionType.EXPERIMENTAL

    return None


def get_latest_version(version_type: VersionType, repo_dir) -> str:
    """Gets the latest version of a certain type.

    Args:
        version_type (VersionType): type of version to get
        repo_dir (str): directory of the repository

    Returns:
        str: latest version as a string
    """

    update_tags(repo_dir)

    out = subprocess.run('git tag --list',
                         check=False,
                         cwd=repo_dir,
                         capture_output=True)

    tags = out.stdout.decode('utf-8').splitlines()

    tag = None

    for tag in reversed(tags):
        if validate_version(tag, version_type, edition=EditionType.BEDROCK):
            break

    tabbed_print(
        texts.VERSION_LATEST_IS.format(version_type=version_type.value,
                                       latest_version="" + tag))
    return tag


def clone_repo() -> str:
    """Clones the mojang/bedrock-samples repository.

    Returns:
        str: path of the repository
    """

    tabbed_print(texts.FILE_DOWNLOADING)

    repo_dir = f'{TEMP_PATH}/bedrock-samples/'

    rm_if_exists(repo_dir)

    clone_command = ['git', 'clone', REPO_URL, repo_dir]

    try:
        subprocess.run(clone_command,
                       check=True,
                       stdout=subprocess.DEVNULL,
                       stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as err:
        print(
            texts.ERROR_COMMAND_FAILED.format(error_code=err.returncode,
                                              error_msg=err.stderr))

    return repo_dir


def update_tags(repo_dir: str):
    """Updates the tags of the repository.

    Args:
        repo_dir (str): directory of the repository
    """
    subprocess.run(
        'git fetch --tags',
        check=False,
        cwd=repo_dir,
    )


def change_repo_version(repo_dir: str, version: str, fetch_tags: bool = True):
    """Changes the version of the repository.

    Args:
        repo_dir (str): directory of the repository
        version (str): version to change to
        fetch_tags (bool, optional): whether to fetch tags from the repository. Defaults to True.

    Raises:
        Exception: if the command fails
    """
    if fetch_tags:
        update_tags(repo_dir)
    try:
        subprocess.run(f'git checkout tags/v{version}',
                       check=False,
                       cwd=repo_dir,
                       stdout=subprocess.DEVNULL,
                       stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as err:
        print(
            texts.ERROR_COMMAND_FAILED.format(error_code=err.returncode,
                                              error_msg=err.stderr))


def get_textures(version_or_type: VersionType | str = VersionType.RELEASE,
                 output_dir=DEFAULT_OUTPUT_DIR,
                 scale_factor=DEFAULT_SCALE_FACTOR,
                 do_merge=True) -> str:
    """Easily extract, filter, and scale item and block textures.

    Args:
        version_or_type (string): a Minecraft Bedrock version, for example "v1.20.0.1" or "v1.20.10.21-preview"
        output_dir (str, optional): directory that the final textures will go. Defaults to `DEFAULT_OUTPUT_DIR`.
        scale_factor (int, optional): factor that will be used to scale the textures. Defaults to `DEFAULT_SCALE_FACTOR`.
        do_merge (bool, optional): whether to merge the block and item textures into a single directory. Defaults to True.

    Returns:
        string: path of the final textures
    """

    if isinstance(version_or_type, str) and not validate_version(
            version_or_type, edition=EditionType.BEDROCK):
        print(texts.VERSION_INVALID.format(version=version_or_type))
        return None

    version_type = version_or_type if isinstance(version_or_type,
                                                 VersionType) else None
    version = None
    asset_dir = clone_repo()
    if isinstance(version_or_type, str):
        version = version_or_type
    else:
        version = get_latest_version(version_type, asset_dir)

    change_repo_version(asset_dir, version)
    tabbed_print(texts.VERSION_USING_X.format(version=version))

    filtered = filter_unwanted(asset_dir,
                               f'{output_dir}/bedrock/{version[1:]}',
                               edition=EditionType.BEDROCK)
    scale_textures(filtered, scale_factor, do_merge)

    tabbed_print(texts.CLEARING_TEMP)
    rm_if_exists(TEMP_PATH)

    output_dir = os.path.abspath(filtered).replace('\\', '/')
    print(texts.COMPLETED.format(output_dir=output_dir))
    return output_dir


def main():
    get_textures(VersionType.RELEASE, scale_factor=100)
    get_textures(VersionType.EXPERIMENTAL, scale_factor=100)


if __name__ == '__main__':
    main()



from .bedrock import get_version_type, get_latest_version, clone_repo, get_textures
from .java import get_version_type, get_version_manifest, get_latest_version, download_client_jar, extract_textures, get_textures
from .common import VersionType, EditionType, validate_version



import sys
from textureminer.common import DEFAULT_SCALE_FACTOR, EditionType, VersionType, tabbed_print
from textureminer import java, bedrock, texts

DEFAULT_VERSION = VersionType.RELEASE
DEFAULT_EDITION = EditionType.JAVA


def get_textures(
    *args,
    edition: EditionType = EditionType.JAVA,
    **kwargs,
) -> str:
    """Easily extract, filter, and scale item and block textures.

    Args:
        *args: arguments that will be passed to the get_textures function of the specified edition
        edition (EditionType): type of edition, defaults to `EditionType.JAVA`
        version_or_type (string): a Minecraft Java version, for example "1.11" or "22w11a"
        scale_factor (int): factor that will be used to scale the textures
        **kwargs: keyword arguments that will be passed to the get_textures function of the specified edition

    Returns:
        string: path of the final textures
    """

    if edition == EditionType.JAVA:
        tabbed_print(texts.EDITION_USING_X.format(edition=edition.value))
        return java.get_textures(*args, **kwargs)

    if edition == EditionType.BEDROCK:
        tabbed_print(texts.EDITION_USING_X.format(edition=edition.value))
        return bedrock.get_textures(*args, **kwargs)
    return None


def cli():
    """CLI entrypoint.

    Returns:
        str: path to the output directory
    """
    args = sys.argv[1:]

    if len(args) > 0 and args[0].lower() in [
            '--help',
            '-h'
            'help',
    ]:
        print(texts.COMMAND_SYNTAX)
        return None

    print(texts.TITLE)

    if len(args) == 0:
        tabbed_print(texts.EDITION_USING_DEFAULT)
        return get_textures(edition=DEFAULT_EDITION,
                            version_or_type=DEFAULT_VERSION,
                            scale_factor=DEFAULT_SCALE_FACTOR)

    version = args[0].lower()

    release = ['stable'
               'release']
    experimental = [
        'experimental', 'snapshot', 'pre-release', 'release candidate',
        'preview'
    ]

    if version in release:
        version = VersionType.RELEASE
    if version in experimental:
        version = VersionType.EXPERIMENTAL

    if len(args) == 1:
        tabbed_print(texts.EDITION_USING_DEFAULT)
        return get_textures(version_or_type=version,
                            edition=DEFAULT_EDITION,
                            scale_factor=DEFAULT_SCALE_FACTOR)

    edition = args[1].lower()

    if edition == EditionType.JAVA.value.lower() and java.validate_version(
            version):
        return get_textures(version_or_type=version,
                            edition=EditionType.JAVA,
                            scale_factor=DEFAULT_SCALE_FACTOR)
    if edition == EditionType.BEDROCK.value.lower(
    ) and bedrock.validate_version(version):
        return get_textures(version_or_type=version,
                            edition=EditionType.BEDROCK,
                            scale_factor=DEFAULT_SCALE_FACTOR)

    tabbed_print(texts.INVALID_COMBINATION)
    tabbed_print(texts.EDITION_USING_DEFAULT)
    return get_textures(version_or_type=DEFAULT_VERSION,
                        edition=DEFAULT_EDITION)


cli()



from enum import Enum
import os
import re
from shutil import copytree, rmtree
import stat
import tempfile
from PIL import Image as pil_image
from forfiles import image, file as f
from textureminer import texts

HOME_DIR = os.path.expanduser('~').replace('\\', '/')
TEMP_PATH = f'{tempfile.gettempdir()}/texture_miner'.replace('\\', '/')

DEFAULT_OUTPUT_DIR = os.path.normpath(f'{HOME_DIR}/Downloads/textures')
DEFAULT_SCALE_FACTOR = 100

REGEX_BEDROCK_RELEASE = r'^v1\.[0-9]{2}\.[0-9]{1,2}\.[0-9]{1,2}$'
REGEX_BEDROCK_PREVIEW = r'^v1\.[0-9]{2}\.[0-9]{1,2}\.[0-9]{1,2}-preview$'

REGEX_JAVA_SNAPSHOT = r'^[0-9]{2}w[0-9]{2}[a-z]$'
REGEX_JAVA_PRE = r'^[0-9]\.[0-9]+\.?[0-9]+-pre[0-9]?$'
REGEX_JAVA_RC = r'^[0-9]\.[0-9]+\.?[0-9]+-rc[0-9]?$'
REGEX_JAVA_RELEASE = r'^[0-9]\.[0-9]+\.?[0-9]+?$'


class VersionType(Enum):
    """Enum class representing different types of versions for Minecraft
    """

    EXPERIMENTAL = 'experimental'
    """snapshot, pre-release, release candidate, or preview
    """
    RELEASE = 'release'
    """stable release
    """


class EditionType(Enum):
    """Enum class representing different editions of Minecraft
    """

    BEDROCK = 'bedrock'
    """Bedrock Edition
    """
    JAVA = 'java'
    """Java Edition
    """


def on_rm_error(func, path, exc_info):
    """Removes read-only files on Windows.

    Args:
        func (function): Function that raised the exception
        path (str): Path of the file that couldn't be deleted
        exc_info (exception): Exception info
    """
    os.chmod(path, stat.S_IWRITE)
    os.unlink(path)


def rm_if_exists(path: str):
    """Removes a file or directory if it exists.

    Args:
        path (str): path of the file or directory that will be removed
    """
    if os.path.exists(path):
        rmtree(path, onerror=on_rm_error)


def tabbed_print(text):
    """Prints a message to the console with cyan text and a bullet point.
    """
    print(f"{texts.STYLED_TAB}{text}")


def mk_dir(path: str, do_delete_prev: bool = False):
    """Makes a directory if one does not already exist.

    Args:
        path (str): path of the directory that will be created
        do_delete_prev (bool): whether to delete the previous directory if it exists
    """
    if do_delete_prev and os.path.isdir(path):
        rmtree(path)

    if not os.path.isdir(path):
        os.makedirs(path)


def validate_version(version: str,
                     version_type: VersionType = None,
                     edition: EditionType = None) -> bool:
    """Validates a version string based on the version type using regex.

    Args:
        version (str): version string to validate
        version_type (VersionType): type of version, defaults to None, which will validate any version
        edition (EditionType): type of edition, defaults to None, which will validate any version

    Returns:
        bool: whether the version is valid
    """

    if edition == EditionType.BEDROCK.value:
        if version[0] != 'v':
            version = f'v{version}'
        if version_type is None:
            return re.match(REGEX_BEDROCK_RELEASE, version) or re.match(
                REGEX_BEDROCK_PREVIEW, version)
        if version_type == VersionType.RELEASE:
            return re.match(REGEX_BEDROCK_RELEASE, version)
        if version_type == VersionType.EXPERIMENTAL:
            return re.match(REGEX_BEDROCK_PREVIEW, version)

    if edition == EditionType.JAVA.value:
        if version_type is None:
            return re.match(REGEX_JAVA_RELEASE, version) or re.match(
                REGEX_JAVA_SNAPSHOT, version) or re.match(
                    REGEX_JAVA_PRE, version) or re.match(
                        REGEX_JAVA_RC, version)
        if version_type == VersionType.RELEASE:
            return re.match(REGEX_JAVA_RELEASE, version)
        if version_type == VersionType.EXPERIMENTAL:
            return re.match(REGEX_JAVA_SNAPSHOT, version) or re.match(
                REGEX_JAVA_PRE, version) or re.match(REGEX_JAVA_RC, version)

    is_valid = re.match(REGEX_BEDROCK_PREVIEW, version) or re.match(
        REGEX_BEDROCK_RELEASE,
        version) or re.match(REGEX_JAVA_RELEASE, version) or re.match(
            REGEX_JAVA_SNAPSHOT, version) or re.match(
                REGEX_JAVA_PRE, version) or re.match(REGEX_JAVA_RC, version)

    if is_valid:
        return True

    if version[0] != 'v':
        version = f'v{version}'

    return re.match(REGEX_BEDROCK_PREVIEW, version) or re.match(
        REGEX_BEDROCK_RELEASE,
        version) or re.match(REGEX_JAVA_RELEASE, version) or re.match(
            REGEX_JAVA_SNAPSHOT, version) or re.match(
                REGEX_JAVA_PRE, version) or re.match(REGEX_JAVA_RC, version)


def filter_unwanted(input_dir: str,
                    output_dir: str,
                    edition: EditionType = EditionType.JAVA) -> str:
    """Removes files that are not item or block textures.

    Args:
        input_path (string): directory where the input files are
        output_path (string): directory where accepted files will end up
        edition (EditionType): type of edition, defaults to `EditionType.JAVA`
    """

    mk_dir(output_dir, do_delete_prev=True)

    blocks_input = f'{input_dir}/block' if edition.value == EditionType.JAVA.value else f'{input_dir}/resource_pack/textures/blocks'
    items_input = f'{input_dir}/item' if edition.value == EditionType.JAVA.value else f'{input_dir}/resource_pack/textures/items'

    blocks_output = f'{output_dir}/blocks'
    items_output = f'{output_dir}/items'

    copytree(blocks_input, blocks_output)
    copytree(items_input, items_output)

    f.filter(blocks_output, ['.png'])
    f.filter(items_output, ['.png'])

    return output_dir


def merge_dirs(input_dir: str, output_dir: str):
    """Merges block and item textures to a single directory.
    Item textures are given priority when there are conflicts.

    Args:
        input_dir (string): directory in which there are subdirectories 'block' and 'item'
        output_dir (string): directory in which the files will be merged into
    """

    block_folder = f'{input_dir}/blocks'
    item_folder = f'{input_dir}/items'

    tabbed_print(texts.TEXTURES_MERGING)
    copytree(block_folder, output_dir, dirs_exist_ok=True)
    rmtree(block_folder)
    copytree(item_folder, output_dir, dirs_exist_ok=True)
    rmtree(item_folder)


def scale_textures(path: str,
                   scale_factor: int = 100,
                   do_merge: bool = True,
                   crop: bool = True) -> str:
    """Scales textures within a directory by a factor

    Args:
        path (string): path of the textures that will be scaled
        scale_factor (int): factor that the textures will be scaled by
        do_merge (bool): whether to merge block and item texture files into a single directory
        crop (bool): whether to crop non-square textures to be square

    Returns:
        string: path of the scaled textures
    """

    if do_merge:
        merge_dirs(path, path)
    tabbed_print(texts.TEXTURES_FILTERING)
    for subdir, _, files in os.walk(path):
        f.filter(f'{os.path.abspath(subdir)}', ['.png'])

        if scale_factor != 1 and len(files) > 0:
            tabbed_print(
                texts.TEXTURES_RESIZING_AMOUNT.format(texture_amount=len(files))
                if do_merge else texts.TEXTURES_RESISING_AMOUNT_IN_DIR.
                format(len(files), os.path.basename(subdir)))

        for fil in files:
            image_path = os.path.normpath(f"{os.path.abspath(subdir)}/{fil}")
            if crop:
                with pil_image.open(image_path) as img:
                    if img.size[0] > 16 or img.size[1] > 16:
                        img = img.crop((0, 0, 16, 16))
                        img.save(image_path)

            if scale_factor != 1:
                image.scale(image_path, scale_factor, scale_factor)

    return path



import os
import re
from shutil import copytree, rmtree
from zipfile import ZipFile
import urllib.request
import requests
from textureminer import texts
from textureminer.common import DEFAULT_OUTPUT_DIR, DEFAULT_SCALE_FACTOR, REGEX_JAVA_PRE, REGEX_JAVA_RC, REGEX_JAVA_RELEASE, REGEX_JAVA_SNAPSHOT, EditionType, VersionType, filter_unwanted, mk_dir, rm_if_exists, tabbed_print, TEMP_PATH, scale_textures, validate_version

VERSION_MANIFEST = None
VERSION_MANIFEST_URL = 'https://piston-meta.mojang.com/mc/game/version_manifest_v2.json'


def get_version_type(version: str) -> VersionType:
    """Gets the type of a version based on regex.

    Args:
        version (str): version to get the type of

    Returns:
        VersionType: type of version
    """
    if version[0] != 'v':
        version = f'v{version}'
    if re.match(REGEX_JAVA_RELEASE, version):
        return VersionType.RELEASE
    if re.match(REGEX_JAVA_SNAPSHOT, version) or re.match(
            REGEX_JAVA_PRE, version) or re.match(REGEX_JAVA_RC, version):
        return VersionType.EXPERIMENTAL
    return None


def get_version_manifest() -> dict:
    """Fetches the version manifest from Mojang's servers.
    If the manifest has already been fetched, it will return the cached version.

    Returns:
        dict: version manifest
    """
    return requests.get(
        VERSION_MANIFEST_URL,
        timeout=10).json() if VERSION_MANIFEST is None else VERSION_MANIFEST


def get_latest_version(version_type: VersionType) -> str:
    """Gets the latest version of a certain type.

    Args:
        version_type (VersionType): type of version to get

    Raises:
        Exception: if the version number is invalid

    Returns:
        str: latest version as a string
    """
    tabbed_print(
        texts.VERSION_LATEST_FINDING.format(version_type=version_type.value))
    latest_version = get_version_manifest()['latest'][version_type.value]
    if not validate_version(
            latest_version, version_type, edition=EditionType.JAVA):

        raise Exception(
            texts.VERSION_INVALID.format(version="" + latest_version))
    tabbed_print(
        texts.VERSION_LATEST_IS.format(version_type=version_type.value,
                                       latest_version="" + latest_version))
    return latest_version


def download_client_jar(
    version: str,
    download_dir: str = f'{TEMP_PATH}/version-jars',
) -> str:
    """Downloads the client .jar file for a specific version from Mojang's servers.

    Args:
        version (str): version to download
        download_dir (str, optional): directory to download the file to

    Returns:
        str: path of the downloaded file
    """

    for v in get_version_manifest()['versions']:
        if v['id'] == version:
            url = v['url']
            break
        else:
            url = None

    json = requests.get(url, timeout=10).json()
    client_jar_url = json['downloads']['client']['url']

    mk_dir(download_dir)
    tabbed_print(texts.FILE_DOWNLOADING)
    urllib.request.urlretrieve(client_jar_url, f'{download_dir}/{version}.jar')
    return f'{download_dir}/{version}.jar'


def extract_textures(
        input_path: str,
        output_path: str = f'{TEMP_PATH}/extracted-textures') -> str:
    """Extracts textures from .jar file.

    Args:
        input_path (str): path of the .jar file
        output_path (str, optional): path of the output directory

    Returns:
        str: path of the output directory
    """

    with ZipFile(input_path, 'r') as zip_object:
        file_amount = len(zip_object.namelist())
        tabbed_print(texts.FILES_EXTRACTING.format(file_amount=file_amount))
        zip_object.extractall(f'{TEMP_PATH}/extracted-files/')
    rmtree(f'{TEMP_PATH}/version-jars/')

    if os.path.isdir(output_path):
        rmtree(output_path)

    copytree(f'{TEMP_PATH}/extracted-files/assets/minecraft/textures',
             output_path)
    rmtree(f'{TEMP_PATH}/extracted-files/')

    return output_path


def get_textures(version_or_type: VersionType | str = VersionType.RELEASE,
                 output_dir=DEFAULT_OUTPUT_DIR,
                 scale_factor=DEFAULT_SCALE_FACTOR,
                 do_merge=True) -> str:
    """Easily extract, filter, and scale item and block textures.

    Args:
        version_or_type (string): a Minecraft Java version, for example "1.11" or "22w11a"
        output_dir (str, optional): directory that the final textures will go. Defaults to `DEFAULT_OUTPUT_DIR`.
        scale_factor (int, optional): factor that will be used to scale the textures. Defaults to `DEFAULT_SCALE_FACTOR`.
        do_merge (bool, optional): whether to merge the block and item textures into a single directory. Defaults to True.

    Returns:
        string: path of the final textures
    """

    if isinstance(version_or_type, str) and not validate_version(
            version_or_type, edition=EditionType.JAVA):
        print(texts.VERSION_INVALID.format(version=version_or_type))
        return None

    version_type = version_or_type if isinstance(version_or_type,
                                                 VersionType) else None

    version = None
    if isinstance(version_or_type, str):
        version = version_or_type
    else:
        version = get_latest_version(version_type)

    tabbed_print(texts.VERSION_USING_X.format(version=version))
    assets = download_client_jar(version)
    extracted = extract_textures(assets)
    filtered = filter_unwanted(extracted,
                               f'{output_dir}/java/{version}',
                               edition=EditionType.JAVA)
    scale_textures(filtered, scale_factor, do_merge)

    tabbed_print(texts.CLEARING_TEMP)
    rm_if_exists(TEMP_PATH)

    output_dir = os.path.abspath(filtered).replace('\\', '/')
    print(texts.COMPLETED.format(output_dir=output_dir))
    return output_dir


def main():
    get_textures(VersionType.RELEASE, scale_factor=100)
    get_textures(VersionType.EXPERIMENTAL, scale_factor=100)


if __name__ == '__main__':
    main()



from fortext import style, Fg

TITLE = style(r"""

 _______ ________   _________ _    _ _____  ______   __  __ _____ _   _ ______ _____
|__   __|  ____\ \ / /__   __| |  | |  __ \|  ____| |  \/  |_   _| \ | |  ____|  __ \
   | |  | |__   \ V /   | |  | |  | | |__) | |__    | \  / | | | |  \| | |__  | |__) |
   | |  |  __|   > <    | |  | |  | |  _  /|  __|   | |\/| | | | | . ` |  __| |  _  /
   | |  | |____ / . \   | |  | |__| | | \ \| |____  | |  | |_| |_| |\  | |____| | \ \
   |_|  |______/_/ \_\  |_|   \____/|_|  \_\______| |_|  |_|_____|_| \_|______|_|  \_\

""",
              fg=Fg.CYAN)
STYLED_TAB = style(f"{' '*4}* ", fg=Fg.CYAN)

COMPLETED = style("Completed. You can find the textures on:\n{output_dir}\n",
                  fg=Fg.GREEN)

COMMAND_SYNTAX = """\n\tpy -m textureminer <version> <edition>
\te.g.
\tpy -m textureminer 1.17 java
\tpt -m textureminer 1.17 bedrock
\tdefaults to java if no edition is specified, and defaults to latest stable if no version is specified"""
EDITION_INVALID = "Invalid edition"
EDITION_USING_DEFAULT = "Using default edition (Java Edition)"
EDITION_USING_X = "Using {edition} Edition"
ERROR_COMMAND_FAILED = "The command failed with return code {error_code}: {error_msg}"
FILE_DOWNLOADING = "Downloading assets..."
FILES_EXTRACTING = "Extracting {file_amount} files..."
CLEARING_TEMP = "Clearing temporary files..."
INVALID_COMBINATION = "Invalid combination of version and edition"
TEXTURES_FILTERING = "Filtering textures..."
TEXTURES_MERGING = "Merging block and item textures to a single directory..."
TEXTURES_RESISING_AMOUNT_IN_DIR = "Resizing {texture_amount} {dir_name} textures..."
TEXTURES_RESIZING_AMOUNT = "Resizing {texture_amount} textures..."
VERSION_INVALID = "Invalid version ({version})"
VERSION_LATEST_FINDING = "Finding latest {version_type} version..."
VERSION_LATEST_IS = "Latest {version_type} is {latest_version}."
VERSION_USING_X = "Using {version} version"
